/*!
 * Plugin:  Skyscraper Menu (jQuery)
 *          Inspired by vertical menu at http://www.cdu-ilmkreis.de/
 * Author:  bernddermeister (bernddermeister AT visualgaze DOT de)
 *          http://www.visualgaze.de
 * Date:    2009/12/07
 * Version: 0.9.2
 * License: under MIT License
 *
 * Usage:   After including jQuery,
 *          jQuery('#menu_id').skyscraperMenu();
 * Options: see index.html
 */

(function($) {
	$.fn.skyscraperMenu = function(options) {
		/*Predefine settings*/
		var settings = jQuery.extend( {
			activeMenuItem: 0,		// 0 sets HOME
			activeMenuItemSub: 0,
			readjustMenu: true,		// rearrange width of submenu entries to full menu width - otherwise use css definitions instead
			readjustFullWidth: true,
			speedIn: 400, animIn: 'easeOutBounce',
			speedOut: 500, animOut: 'easeOutBounce',
			activeClass: (jQuery.browser.msie && jQuery.browser.version <= 6) ? 'activeIE' : 'active'
		}, options );

		settings.activeMenuItemOffset = settings.activeMenuItem-1;
		settings.activeMenuItemSubOffset = settings.activeMenuItemSub-1;
		/*Current item(s)*/
		var item = jQuery(this);
		if (item.length==0) { alert("Can't find menu ID !"); }
		else {
			var childs = item.children('UL.hMenuItem');
			if (childs.length==0) { alert("No menu found !"); }
			else {
				initMenu();
				/*Observe events*/
				childs.mouseenter(
					function() {
						jQuery('#hMenuHover').stop(false, true).slideDown({ duration: settings.speedIn, easing: settings.animIn });
						item.find('UL.hMenuItemSub').stop(false, true).slideDown({ duration: settings.speedIn, easing: settings.animIn });
						setSlider(jQuery(this).parent().children('UL').index(this));
					})
				.children('LI').children('UL.hMenuItemSub').children('LI')
					.find('a')
					.filter(function() { return (jQuery(this).hasClass(settings.activeClass))?false:true; })
					.mouseenter(function() {jQuery(this).addClass(settings.activeClass);})
					.mouseleave(function() {jQuery(this).removeClass(settings.activeClass);});

				item.mouseleave(function() { updateMenu(); });
			}
		}

		/*Initialize menu*/
		function initMenu() {
			item.prepend('<div id="hMenuBackground"></div>');	// prepend menu background element (image)
			item.prepend('<div id="hMenuMain"></div>');				// prepend main menu background
			item.prepend('<div id="hMenuHover"></div>');			// prepend background hover effect
			item.prepend('<div id="hMenuSlider"></div>');			// prepend slider

			// set positions of sub menu cols
			if (settings.readjustMenu) {
				var last = -110;
				jQuery.each(childs, function(i, val) {
					var subLinks = jQuery(this).find('a');
					if (settings.readjustFullWidth) {
						// set width and position of sub menus
						var brdLftRgt = parseInt(jQuery('#hMenuSlider').css('border-left-width')) + parseInt(jQuery('#hMenuSlider').css('border-right-width'));
						var iWidth = (parseInt(item.width())+2)/childs.length;
						jQuery(this).css({width: (iWidth-brdLftRgt)+'px', left: (i*iWidth)+'px'});
						// set width and position of sub menu links
						var padLftRgt = parseInt(subLinks.css('padding-left')) + parseInt(subLinks.css('padding-right'));
						subLinks.css({width: (iWidth-(padLftRgt+brdLftRgt))+'px', left: (i*iWidth)+'px'});
						// set slider width
						jQuery('#hMenuSlider').css({width: (item.width()/childs.length)});
					} else {
						// set width of sub menu links
						var brdLftRgt = parseInt(jQuery(this).children('LI').children('DIV.hMenuItemBackground').css('border-left-width')) + parseInt(jQuery(this).children('LI').children('DIV.hMenuItemBackground').css('border-right-width'));
						var padLftRgt = parseInt(subLinks.css('padding-left')) + parseInt(subLinks.css('padding-right'));
						subLinks.css({width: (jQuery(this).width())-(padLftRgt+brdLftRgt)});
					}
				});
			}
			setActiveEntry(); // set active sub entry
		}

		/*Hide all sub menus and show the current*/
		function updateMenu() {
			// hide full background hilight
			jQuery('#hMenuHover').stop(false, true).slideUp({ duration: settings.speedOut, easing: settings.animOut });
			// hide slider
			jQuery('#hMenuSlider').slideUp(100);
			// hide all sub menus except active
			item.find('.hMenuItemSub:lt('+settings.activeMenuItemOffset+'), .hMenuItemSub:gt('+settings.activeMenuItemOffset+')').stop(false, true).slideUp({ duration: settings.speedOut, easing: settings.animOut });
			item.find('.hMenuItemButton:lt('+settings.activeMenuItemOffset+'), .hMenuItemButton:gt('+settings.activeMenuItemOffset+')').find('a').removeClass(settings.activeClass);
		}

		/*Set the Slider background*/
		function setSlider(index) {
			var subs = childs.eq(index).children('LI').children('UL.hMenuItemSub').find('LI').children('A');
			var padTopBtm = (subs.length > 0) ? parseInt(subs.css('padding-top')) + parseInt(subs.css('padding-bottom')) : 10;
			var mainHeight = parseInt(childs.eq(index).children('LI').children('DIV.hMenuItemButton').css('height'));
			jQuery('#hMenuSlider').stop(false, true).animate({
				left: parseInt(childs.eq(index).css('left'))+'px',
				width: parseInt(childs.eq(index).css('width'))+'px',
				height: ( (((subs.length > 0)?parseInt(subs.css('height')):15)+padTopBtm)*subs.length)+(padTopBtm+mainHeight)+'px'
			}, 100);
			childs.filter(function() {return (jQuery(this).parent().children('UL').index(this) != settings.activeMenuItemOffset); }).find('DIV.hMenuItemButton').find('a').removeClass(settings.activeClass);
			childs.eq(index).find('DIV.hMenuItemButton').find('a').addClass(settings.activeClass);
		}

		/*Set active sub entry*/
		function setActiveEntry() {
			if (settings.activeMenuItem != 0 && settings.activeMenuItemSub != 0) {
				// get active menu item
				var activeElem = jQuery(childs).eq(settings.activeMenuItemOffset);
				// set active main menu entry
				activeElem.children('LI').children('DIV.hMenuItemButton').find('a').addClass(settings.activeClass);
				// show active sub menu
				activeElem.children('LI').children('UL.hMenuItemSub').show();
				// set active sub menu background and height
				var subLinks = activeElem.children('LI').children('UL.hMenuItemSub').children('LI').find('a');
				var padTopBtm = parseInt(subLinks.css('padding-top')) + parseInt(subLinks.css('padding-bottom'));
				var mainHeight = parseInt(activeElem.children('LI').children('DIV.hMenuItemButton').css('height'));
				activeElem.children('LI').children('DIV.hMenuItemBackground')
					.css('height', ((parseInt(subLinks.css('height'))+padTopBtm)*subLinks.length)+(padTopBtm+mainHeight)+'px')
					.show();
				// add class to active sub menu entry
				var curElem = activeElem.children('LI').children('UL.hMenuItemSub').find('LI:eq('+settings.activeMenuItemSubOffset+')').children('A');
				if (curElem.length != 0)
					curElem.addClass(settings.activeClass);
				else
					alert('subentry '+settings.activeMenuItemSub+' not found in active sub menu '+settings.activeMenuItem);
			}
		}

	}
})(jQuery);
